/***************************************************************************
                          kslidesaver.cpp  -  description
                             -------------------
    begin                : Fri Feb 21 00:14:47 CST 2003
    copyright            : (C) 2003 by Wojciech Jarosz
    email                : wjarosz@renderbtch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "kslidesaver.h"
#include <qapp.h>
#include <iostream>
#include <kconfig.h>
#include <qtimer.h>
#include <qdir.h>
#include <time.h>
#include "math/utility.h"
#include "math/random.h"
#include "math/common.h"
#include "math/imageloader.h"
#include "math/jpegloader.h"
#include "math/pngloader.h"
#include "math/byteraster.h"
#include <qdatetime.h>
//#include <algorithm>
#include <string>

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glx.h>

/*!
	For some reason this font stuff has to be outside of the class.
	If I have it in the class it segfaults when being embeded in the
	setup window. Still haven't figured out why, probably something
	to do with the way that Qt handles GL Contexts.
*/
GLuint fontBase;            //!< base display list for the font set.

GLvoid BuildFont(GLvoid)
{
	Display *dpy;
	XFontStruct *fontInfo;			// storage for our font.

	fontBase = glGenLists(96);		// storage for 96 characters.

	// load the font.  what fonts any of you have is going
	// to be system dependent. in any case, one of these 2
	// fonts should be on your system...or you won't see any text.

	// get the current display.  This opens a second
	// connection to the display in the DISPLAY environment
	// value, and will be around only long enough to load
	// the font.
	dpy = XOpenDisplay(NULL);		// default to DISPLAY env.

	fontInfo = XLoadQueryFont(dpy, "-adobe-helvetica-medium-r-normal--10-*-*-*-p-*-iso8859-1");
	if (fontInfo == NULL)
	{
		fontInfo = XLoadQueryFont(dpy, "fixed");
		if (fontInfo == NULL)
		{
			printf("no X font available?\n");
		}
	}

    // after loading this font info, this would probably be the time
    // to rotate, scale, or otherwise twink your fonts.

    // start at character 32 (space), get 96 characters (a few characters past z), and
    // store them starting at fontBase.
    glXUseXFont(fontInfo->fid, 32, 96, fontBase);

    // free that font's info now that we've got the
    // display lists.
    XFreeFont(dpy, fontInfo);

    // close down the 2nd display connection.
    XCloseDisplay(dpy);
}


GLvoid KillFont(GLvoid)
{
    glDeleteLists(fontBase, 96);
}

GLvoid glPrint(const char *text)
{
	if (text == NULL)
	{	// if there's no text, do nothing.
		return;
	}

	glPushAttrib(GL_LIST_BIT);
	{
		// sets the base character to 32.
		glListBase(fontBase - 32);

		// draws the display list text.
		glCallLists(strlen(text), GL_UNSIGNED_BYTE, text);
	}
	glPopAttrib();
}

KSlideSaver::KSlideSaver(QWidget *parent, const char *name)
	: QGLWidget(parent,name),
	idle_timer(NULL),
	opacity1(1.0),
	opacity2(1.0),
	glImage1Width(1.0),
	glImage1Height(1.0),
	glImage2Width(1.0),
	glImage2Height(1.0),
	redrawWait(15),
	reloadWait(10000),
	startFade(.75),
	image1Show(false),
	image2Show(false),
	imageL1(NULL),
	imageL2(NULL),
	image1First(true),
	loadingImage1(false),
	loadingImage2(false),
	image1Index(0),
	image2Index(1),
	randomize(true),
	starting(true),
	w1Ratio(1.0),
	w2Ratio(1.0),
	h1Ratio(1.0),
	h2Ratio(1.0),
	oldW1(-1),
	oldH1(-1),
	oldW2(-1),
	oldH2(-1),
	windowAspect(1.0),
	showFPS(false),
	showImageNames(false),
	fpsTimer(NULL),
	image1Name(),
	image2Name(),
	fps(0.0),
	frameNum(0)
{
	readConfig();
	sgenrand((unsigned int)time((time_t *)NULL));

	readImageDirectory();

	start1 = new QTime();
	start2 = new QTime();
	fpsTimer = new QTime();
	fpsTimer->start();

	idle_timer = new QTimer( this );
	connect( idle_timer, SIGNAL(timeout()), SLOT(update()) );
	std::cout << "hi" << std::endl;
}

void KSlideSaver::init()
{
	idle_timer = NULL;
	opacity1 = 1.0;
	opacity2 = 1.0;
	glImage1Width = 1.0;
	glImage1Height = 1.0;
	glImage2Width = 1.0;
	glImage2Height = 1.0;
	//redrawWait = 15;
	//reloadWait = 10000;
	//startFade = .75;
	image1Show = false;
	image2Show = false;
	imageL1 = NULL;
	imageL2 = NULL;
	image1First = true;
	loadingImage1 = false;
	loadingImage2 = false;
	image1Index = 0;
	image2Index = 1;
	//randomize = true;
	starting = true;
	w1Ratio = 1.0;
	w2Ratio = 1.0;
	h1Ratio = 1.0;
	h2Ratio = 1.0;
	oldW1 = -1;
	oldH1 = -1;
	oldW2 = -1;
	oldH2 = -1;
	windowAspect = 1.0;
	//showFPS = false,
	//showImageNames = false,
	fpsTimer = NULL;
	image1Name = QString();
	image2Name = QString();
	fps = 0.0;
	frameNum = 0;
}

KSlideSaver::~KSlideSaver()
{
	delete start1;
	delete start2;
	delete idle_timer;
	delete imageL1;
	delete imageL2;
	delete fpsTimer;
	KillFont();
	glDeleteTextures(2, &texture[0]);
}

void KSlideSaver::loadImage1()
{
	if (!loadingImage1 && fileNames.size())
	{
		loadingImage1 = true;
		QTime now;
		now.start();

		delete imageL1;
		image1Name = QString(fileNames[image1Index % fileNames.size()].c_str());
		imageL1 = imageLoader::inferType(image1Name);

		std::cout << image1Name.latin1() << std::endl;
		//imageL1->read_image(image1Name.latin1());
		if (imageL1->start_incremental_image_read(image1Name.latin1()))
		{	// the image exists and we can read it
			int numLines = 1;
			for (int i = 0; i < imageL1->height(); i+=numLines)
			{
				if (now.elapsed() > 0.5*redrawWait && start2->elapsed() < startFade*reloadWait)
				{
					qApp->processEvents();
					now.restart();
				}
				imageL1->incremental_image_read(numLines);
			}
			imageL1->finish_incremental_image_read();
		}

		if (image1Index+2 >= (int)fileNames.size() && randomize)
		{
			randomizeImages();
		}

		image1Index = (image1Index+2) % fileNames.size();


		bindImage1();
		loadingImage1 = false;
	}
}


void KSlideSaver::loadImage2()
{
	if (!loadingImage2 && fileNames.size())
	{
		loadingImage2 = true;
		QTime now;
		now.start();

		delete imageL2;
		image2Name = QString(fileNames[image2Index % fileNames.size()].c_str());
		imageL2 = imageLoader::inferType(image2Name);//

		std::cout << image2Name.latin1() << std::endl;
		//imageL2->read_image(image2Name.latin1());
		if (imageL2->start_incremental_image_read(image2Name.latin1()))
		{
			int numLines = 1;
			for (int i = 0; i < imageL2->height(); i+=numLines)
			{
				if (now.elapsed() > 0.5*redrawWait && start1->elapsed() < startFade*reloadWait)
				{
					qApp->processEvents();
					now.restart();
				}
				imageL2->incremental_image_read(numLines);
			}
			imageL2->finish_incremental_image_read();
		}

		image2Index = (image2Index+2) % fileNames.size();

		bindImage2();
		loadingImage2 = false;
	}
}

void KSlideSaver::randomizeImages()
{
	// randomly shuffle the array of filenames
	//std::random_shuffle(fileNames.begin(), fileNames.end());

	std::string temp;
	int index = 0;
	for (int i = 0; i < (int)fileNames.size(); i++)
	{
		temp = fileNames[i];
		index = i + int(rB_random()*(fileNames.size()-i));

		fileNames[i] = fileNames[index];
		fileNames[index] = temp;
	}
}

void KSlideSaver::addDirectory(const QString& dir, int depth)
{
	QDir d(dir);

	if ( !d.exists() )
	{
		qWarning(QString("Cannot find the \"") + d.absPath().latin1() + QString("\" directory"));
	}
	else
	{
		d.setFilter(QDir::Files | QDir::Hidden);
		d.setSorting(QDir::Name);

		// create a string which contains all supported image formats
		QStringList fileNames_strings = d.entryList(QString("*.jpg *.JPG *.jpeg *.JPEG *.PNG *.png"));

		// read in all files which can be read in this directory
		QStringList::Iterator fileName_list_it = fileNames_strings.begin();
		while (fileName_list_it != fileNames_strings.end())
		{
			fileNames.push_back(std::string(dir.latin1()) + ((dir.at(dir.length()-1) == "/") ? std::string("") : std::string("/")) + std::string((*fileName_list_it).latin1()));
			++fileName_list_it;
		}

		if (depth == 0)
		{
			// add any subdirectories
			d.setFilter(QDir::Dirs);

			// create a string which contains all supported image formats
			QStringList subDirectory_strings = d.entryList(QString("*"));

			// read in all files which can be read in this directory
			fileName_list_it = subDirectory_strings.begin();
			++fileName_list_it; ++fileName_list_it;
			while (fileName_list_it != subDirectory_strings.end())
			{
				addDirectory(QString(std::string(std::string(dir.latin1())).c_str()) + *fileName_list_it, depth+1);
				++fileName_list_it;
			}
		}
	}
}

void KSlideSaver::readImageDirectory()
{
	if (imageDirectory.at(imageDirectory.length()-1) != '/')
		imageDirectory += "/";

	QDir d(imageDirectory);
	if ( !d.exists() )
	{
		qWarning(QString("Cannot find the \"") + imageDirectory + QString("\" directory"));
	}
	else
	{

		fileNames = std::vector<std::string>();

		addDirectory(imageDirectory,0);

		if (randomize)
			randomizeImages();
	}
}

void KSlideSaver::bindImage1(bool update)
{
	float aspect = (imageL1->height() != 0) ? (float)imageL1->width()/(float)imageL1->height() : 1.0;

	if (aspect < windowAspect)
	{	// image is taller than window, match the image to the window width
		glImage1Width = 2*windowAspect;
		glImage1Height = (aspect != 0) ? glImage1Width/aspect : 0;
	}
	else
	{	// image is wider than window, match the image to the window height
		glImage1Height = 2;
		glImage1Width = glImage1Height*aspect;
	}

	int pow2Width = (1 << (int)ceil(log(imageL1->width())/LOG_2));
	int pow2Height = (1 << (int)ceil(log(imageL1->height())/LOG_2));

	w1Ratio = (float)imageL1->width()/pow2Width;
	h1Ratio = (float)imageL1->height()/pow2Height;

	if (rB_random() < 0.5)
	{
		startLoc1 = Vec3(	lerp(-(glImage1Width-2*windowAspect), (glImage1Width-2*windowAspect), 2*rB_random()-1.0),
							lerp(-(glImage1Height-2), 0.0, rB_random()),
							2);

		endLoc1 = Vec3(	clamp(startLoc1[0]+windowAspect*(2*rB_random()-1.0), -(glImage1Width-2*windowAspect), (glImage1Width-2*windowAspect)),
						clamp(startLoc1[1]+2*rB_random()-1.0, -(glImage1Height-2), (glImage1Height-2)),
						1.5);
	}
	else
	{
		endLoc1 = Vec3(	lerp(-(glImage1Width-2*windowAspect), (glImage1Width-2*windowAspect), 2*rB_random()-1.0),
						lerp(-(glImage1Height-2), 0.0, rB_random()),
						2);

		startLoc1 = Vec3(	clamp(endLoc1[0]+windowAspect*(2*rB_random()-1.0), -(glImage1Width-2*windowAspect), (glImage1Width-2*windowAspect)),
							clamp(endLoc1[1]+2*rB_random()-1.0, -(glImage1Height-2), (glImage1Height-2)),
							1.5);
	}

	currentLoc1 = startLoc1;

	if (!imageL1->isNull())
	{
	glBindTexture(GL_TEXTURE_2D, texture[0]);
	if (oldW1 != pow2Width || oldH1 != pow2Height)
	{
		glTexImage2D(GL_TEXTURE_2D, 0, 3, pow2Width, pow2Height, 0, (imageL1->channels() == 3) ? GL_RGB : GL_LUMINANCE, GL_UNSIGNED_BYTE, NULL);
	}

	int step = 32;
	int lines = 32;
	QTime now;
	now.start();
	for (int i = 0; i < imageL1->height(); i+=step)
	{
		if (update && now.elapsed() > 0.5*redrawWait && start2->elapsed() < startFade*reloadWait)
		{
			qApp->processEvents();
			glBindTexture(GL_TEXTURE_2D, texture[0]);
			now.restart();
		}

		lines = (imageL1->height() > i+step) ? step : imageL1->height()-i;
		glTexSubImage2D(GL_TEXTURE_2D, 0, 0, i, imageL1->width(), lines, (imageL1->channels() == 3) ? GL_RGB : GL_LUMINANCE, GL_UNSIGNED_BYTE, imageL1->raster()->pixel(0,i));
	}

	oldW1 = pow2Width;
	oldH1 = pow2Height;
	}
	else
	{	// the image didn't load correctly
		pow2Width = 1;
		pow2Height = 1;
		glBindTexture(GL_TEXTURE_2D, texture[0]);
		if (oldW1 != pow2Width || oldH1 != pow2Height)
		{
			unsigned char nullImage = 0;
			glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE, pow2Width, pow2Height, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, &nullImage);
		}

		w1Ratio = h1Ratio = 1;

		oldW1 = pow2Width;
		oldH1 = pow2Height;
	}

	delete imageL1;
	imageL1 = NULL;

	image1First = true;
}

void KSlideSaver::bindImage2(bool update)
{
	float aspect = (imageL2->height() != 0) ? (float)imageL2->width()/(float)imageL2->height() : 1.0f;

	if (aspect < windowAspect)
	{	// image is taller than window, match the image to the window width
		glImage2Width = 2*windowAspect;
		glImage2Height = (aspect != 0) ? glImage2Width/aspect : 0;
	}
	else
	{	// image is wider than window, match the image to the window height
		glImage2Height = 2;
		glImage2Width = glImage2Height*aspect;
	}

	int pow2Width = (1 << (int)ceil(log(imageL2->width())/LOG_2));
	int pow2Height = (1 << (int)ceil(log(imageL2->height())/LOG_2));

	w2Ratio = (float)imageL2->width()/pow2Width;
	h2Ratio = (float)imageL2->height()/pow2Height;

	if (rB_random() < 0.5)
	{
		startLoc2 = Vec3(	lerp(-(glImage2Width-2*windowAspect), (glImage2Width-2*windowAspect), 2*rB_random()-1.0),
							lerp(-(glImage2Height-2), 0.0, rB_random()),
							2);

		endLoc2 = Vec3(	clamp(startLoc2[0]+windowAspect*(2*rB_random()-1.0), -(glImage2Width-2*windowAspect), (glImage2Width-2*windowAspect)),
						clamp(startLoc2[1]+2*rB_random()-1.0, -(glImage2Height-2), (glImage2Height-2)),
						1.5);
	}
	else
	{
		endLoc2 = Vec3(	lerp(-(glImage2Width-2*windowAspect), (glImage2Width-2*windowAspect), 2*rB_random()-1.0),
						lerp(-(glImage2Height-2), 0.0, rB_random()),
						2);

		startLoc2 = Vec3(	clamp(endLoc2[0]+windowAspect*(2*rB_random()-1.0), -(glImage2Width-2*windowAspect), (glImage2Width-2*windowAspect)),
							clamp(endLoc2[1]+2*rB_random()-1.0, -(glImage2Height-2), (glImage2Height-2)),
							1.5);
	}
	currentLoc2 = startLoc2;

	if (!imageL2->isNull())
	{
	glBindTexture(GL_TEXTURE_2D, texture[1]);
	if (oldW2 != pow2Width || oldH2 != pow2Height)
	{
		glTexImage2D(GL_TEXTURE_2D, 0, 3, pow2Width, pow2Height, 0, (imageL2->channels() == 3) ? GL_RGB : GL_LUMINANCE, GL_UNSIGNED_BYTE, NULL);
	}

	int step = 32;
	int lines = 32;
	QTime now;
	now.start();
	for (int i = 0; i < imageL2->height(); i+=step)
	{
		if (update && now.elapsed() > 0.5*redrawWait && start1->elapsed() < startFade*reloadWait)
		{
			qApp->processEvents();
			glBindTexture(GL_TEXTURE_2D, texture[1]);
			now.restart();
		}

		lines = (imageL2->height() > i+step) ? step : imageL2->height()-i;
		glTexSubImage2D(GL_TEXTURE_2D, 0, 0, i, imageL2->width(), lines, (imageL2->channels() == 3) ? GL_RGB : GL_LUMINANCE, GL_UNSIGNED_BYTE, imageL2->raster()->pixel(0,i));
	}

	oldW2 = pow2Width;
	oldH2 = pow2Height;
	}
	else
	{	// the image didn't load correctly
		pow2Width = 1;
		pow2Height = 1;
		glBindTexture(GL_TEXTURE_2D, texture[1]);
		if (oldW2 != pow2Width || oldH2 != pow2Height)
		{
			unsigned char nullImage = 0;
			glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE, pow2Width, pow2Height, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, &nullImage);
		}

		w2Ratio = h2Ratio = 1;

		oldW2 = pow2Width;
		oldH2 = pow2Height;
	}

	delete imageL2;
	imageL2 = NULL;

	image1First = false;
}

void KSlideSaver::initializeGL()
{
	std::cout << "hi" << std::endl;
	glEnable(GL_TEXTURE_2D);							// Enable Texture Mapping
	glClearColor(0.0, 0.0, 0.0, 0.0f);					// Clear The Background Color
	//glClearDepth(1.0);									// Enables Clearing Of The Depth Buffer
	//glDepthFunc(GL_LESS);								// The Type Of Depth Test To Do
	//glShadeModel(GL_SMOOTH);							// Enables Smooth Color Shading
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);	// Blending Function For Translucency Based On Source Alpha Value ( NEW )
	glEnable(GL_BLEND);									// Turn Blending On
	glDisable(GL_DEPTH_TEST);

	//glMatrixMode(GL_PROJECTION);
	//glLoadIdentity();									// Reset The Projection Matrix

	windowAspect = (float)width()/(float)height();
	//gluPerspective(90, windowAspect, 0.1, 5.0);
	//glMatrixMode(GL_MODELVIEW);

	glPixelStorei(GL_UNPACK_ALIGNMENT, 1); 				// Pixels are byte-aligned

	glGenTextures(2, &texture[0]);

	glBindTexture(GL_TEXTURE_2D, texture[0]);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);

	glBindTexture(GL_TEXTURE_2D, texture[1]);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);

	if (fileNames.size())
	{
		image1Name = QString(fileNames[image1Index % fileNames.size()].c_str());
		imageL1 = imageLoader::inferType(image1Name);
		imageL1->read_image(image1Name.latin1());

		image1Index = (image1Index+2) % fileNames.size();

		bindImage1(false);

		image1Show = true;
		image2Show = false;
	}
	delete imageL1;
	imageL1 = NULL;

	start1->start();
	idle_timer->start(redrawWait, false);

	BuildFont();

}

void KSlideSaver::resizeGL(int w, int h)
{
	if (h==0)						// Prevent A Divide By Zero If The Window Is Too Small
		h=1;

	windowAspect = (float)w/(float)h;

	glViewport(0, 0, w, h);			// Reset The Current Viewport And Perspective Transformation

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective(90, windowAspect, 0.1, 5.0);
	glMatrixMode(GL_MODELVIEW);
}


void KSlideSaver::paintOverlays()
{
	// draw screen overlays
	glDisable(GL_TEXTURE_2D);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0, width(), 0, height());

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	if (showFPS)
	{
		QString buffer;

		int elapsed = fpsTimer->elapsed();
		if (elapsed > 500)
		{
			elapsed = fpsTimer->restart();
			fps = (double)frameNum/(double)elapsed*1000.0;
			frameNum = 0;
		}

		buffer = "fps: " + QString::number(fps);

		glColor4d(0.0, 0.0, 0.0, 0.75);
		glRasterPos2f(11, height()-16);
		glPrint(buffer.latin1());

		glColor4d(1.0, 1.0, 1.0, 0.75);
		glRasterPos2f(10, height()-15);
		glPrint(buffer.latin1());

		frameNum++;
	}

	if (showImageNames)
	{
		float timePercent1 = (float)start1->elapsed()/(float)reloadWait;
		opacity1 = 1.0;
		if (timePercent1 >= 1.0)
		{
			opacity1 = 0.0;
		}
		else if (timePercent1 > startFade)
		{
			opacity1 = (1.0/(1.0 - startFade))*(1.0 - timePercent1);
		}
		else if (timePercent1 < 1.0 - startFade)
		{
			opacity1 = (1.0/(1.0 - startFade))*timePercent1;
		}

		float timePercent2 = (float)start2->elapsed()/(float)reloadWait;
		opacity2 = 1.0;
		if (timePercent2 >= 1.0)
		{
			opacity2 = 0.0;
		}
		else if (timePercent2 > startFade)
		{
			opacity2 = (1.0/(1.0 - startFade))*(1.0 - timePercent2);
		}
		else if (timePercent2 < 1.0 - startFade)
		{
			opacity2 = (1.0/(1.0 - startFade))*timePercent2;
		}

		glColor4d(0.0, 0.0, 0.0, 0.75*opacity1);
		glRasterPos2f(11, 10);
		glPrint(image1Name.latin1());

		glColor4d(1.0, 1.0, 1.0, 0.75*opacity1);
		glRasterPos2f(10, 11);
		glPrint(image1Name.latin1());

		glColor4d(0.0, 0.0, 0.0, 0.75*opacity2);
		glRasterPos2f(11, 10);
		glPrint(image2Name.latin1());

		glColor4d(1.0, 1.0, 1.0, 0.75*opacity2);
		glRasterPos2f(10, 11);
		glPrint(image2Name.latin1());
	}

	glEnable(GL_TEXTURE_2D);
}

void KSlideSaver::drawImage1()
{
	float timePercent = (float)start1->elapsed()/(float)reloadWait;
	currentLoc1 = lerp(startLoc1, endLoc1, timePercent);

	opacity1 = 1.0;

	if (starting)
	{
		starting = false;
		image2Show = false;
		opacity2 = 0.0;
		loadImage2();
	}

	if (timePercent > startFade)
	{
		opacity1 = (1.0/(1.0 - startFade))*(1.0 - timePercent);
		if (!image2Show)
		{
			image2Show = true;
			start2->start();
		}
	}

	if (timePercent >= 1.0)
	{
		image1Show = false;
		opacity1 = 0.0;
		loadImage1();
		return;
	}

	if (!image1Show)
		return;

	glPushMatrix();
	{
		glTranslated(currentLoc1[0], currentLoc1[1], -currentLoc1[2]);
		glScaled(1.0, -1.0, 1.0);

		glBindTexture(GL_TEXTURE_2D, texture[0]);

		glMatrixMode(GL_TEXTURE);
		glPushMatrix();
		{
			glScaled(w1Ratio,h1Ratio,1.0);

			glColor4d(1.0, 1.0, 1.0, opacity1);
			glBegin(GL_QUADS);
			{
				glTexCoord2d(0, 0);
				glVertex3d(-glImage1Width, -glImage1Height, 0);

				glTexCoord2d(1, 0);
				glVertex3d(glImage1Width, -glImage1Height, 0);

				glTexCoord2d(1, 1);
				glVertex3d(glImage1Width, glImage1Height, 0);

				glTexCoord2d(0, 1);
				glVertex3d(-glImage1Width, glImage1Height, 0);
			}
			glEnd();
		}
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
	}
	glPopMatrix();
}

void KSlideSaver::drawImage2()
{
	float timePercent = (float)start2->elapsed()/(float)reloadWait;
	currentLoc2 = lerp(startLoc2, endLoc2, timePercent);

	opacity2 = 1.0;

	if (timePercent > startFade)
	{
		opacity2 = (1.0/(1.0 - startFade))*(1.0 - timePercent);
		if (!image1Show)
		{
			image1Show = true;
			start1->start();
		}
	}

	if (timePercent >= 1.0)
	{
		image2Show = false;
		opacity2 = 0.0;
		loadImage2();
		return;
	}

	if (!image2Show)
		return;

	glPushMatrix();
	{
		glTranslated(currentLoc2[0], currentLoc2[1], -currentLoc2[2]);
		glScaled(1.0, -1.0, 1.0);

		glBindTexture(GL_TEXTURE_2D, texture[1]);

		glMatrixMode(GL_TEXTURE);
		glPushMatrix();
		{
			glScaled(w2Ratio,h2Ratio,1.0);

			glColor4d(1.0, 1.0, 1.0, opacity2);
			glBegin(GL_QUADS);
			{
				glTexCoord2d(0, 0);
				glVertex3d(-glImage2Width, -glImage2Height, 0);

				glTexCoord2d(1, 0);
				glVertex3d(glImage2Width, -glImage2Height, 0);

				glTexCoord2d(1, 1);
				glVertex3d(glImage2Width, glImage2Height, 0);

				glTexCoord2d(0, 1);
				glVertex3d(-glImage2Width, glImage2Height, 0);
			}
			glEnd();
		}
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
	}
	glPopMatrix();
}

void KSlideSaver::drawNoImage()
{
	glDisable(GL_TEXTURE_2D);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0, width(), 0, height());

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	QString buffer = QString("No images in the \"") + imageDirectory + QString("\" directory");

	glColor4d(0.0, 0.0, 0.0, 1.0);
	glRasterPos2f(11, height()-16);
	glPrint(buffer.latin1());

	glColor4d(1.0, 1.0, 1.0, 1.0);
	glRasterPos2f(10, height()-15);
	glPrint(buffer.latin1());

	buffer = QString("Please choose a different image directory.");

	glColor4d(0.0, 0.0, 0.0, 1.0);
	glRasterPos2f(11, height()-26);
	glPrint(buffer.latin1());

	glColor4d(1.0, 1.0, 1.0, 1.0);
	glRasterPos2f(10, height()-25);
	glPrint(buffer.latin1());

	glEnable(GL_TEXTURE_2D);
}


void KSlideSaver::paintGL()
{
	if (fileNames.size())
	{
		glClear(GL_COLOR_BUFFER_BIT);			// Clear The Screen And The Depth Buffer
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();						// Reset The Projection Matrix

		gluPerspective(90, windowAspect, 0.1, 5.0);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();						// Reset The View

		if (image1First)
		{
			if (image1Show)
				drawImage1();
			if (image2Show)
				drawImage2();
		}
		else
		{
			if (image2Show)
				drawImage2();
			if (image1Show)
				drawImage1();
		}

		paintOverlays();
	}
	else
	{
		glClear(GL_COLOR_BUFFER_BIT);
		drawNoImage();
	}
}

void KSlideSaver::readConfig()
{
	KConfig config("kslidesaverglrc",true,false);
	imageDirectory = config.readPathEntry("imageDirectory", "$HOME");
	reloadWait = config.readNumEntry("reloadDelay",10)*1000;
	startFade = 1.0 - lerp(.05,.45,(float)config.readNumEntry("fade",5)/10.0);
	randomize = config.readBoolEntry("randomize",true);
	showFPS = config.readBoolEntry("showFPS",false);
	showImageNames = config.readBoolEntry("showImageNames",false);
}


void KSlideSaver::setDelay(int delay)
{
	reloadWait = delay*1000;
}

void KSlideSaver::setDirectory(const QString& directory)
{
	delete start1;
	delete start2;
	delete idle_timer;
	delete imageL1;
	delete imageL2;
	delete fpsTimer;
	KillFont();
	glDeleteTextures(2, &texture[0]);

	init();

	imageDirectory = directory;
	readImageDirectory();

	start1 = new QTime();
	start2 = new QTime();
	fpsTimer = new QTime();
	fpsTimer->start();

	idle_timer = new QTimer(this);
	connect(idle_timer, SIGNAL(timeout()), SLOT(update()));

	initializeGL();
}

void KSlideSaver::setRandomize(bool b)
{
	if (b != randomize)
	{
		delete start1;
		delete start2;
		delete idle_timer;
		delete imageL1;
		delete imageL2;
		delete fpsTimer;
		KillFont();
		glDeleteTextures(2, &texture[0]);

		init();
		randomize = b;

		readImageDirectory();

		start1 = new QTime();
		start2 = new QTime();
		fpsTimer = new QTime();
		fpsTimer->start();

		idle_timer = new QTimer(this);
		connect(idle_timer, SIGNAL(timeout()), SLOT(update()));

		initializeGL();
	}
}

void KSlideSaver::setFadeAmount(int percent)
{
	startFade = 1.0 - lerp(.05,.45,(float)percent/10.0);
}

void KSlideSaver::setShowFPS(bool b)
{
	showFPS = b;
}

void KSlideSaver::setShowImageNames(bool b)
{
	showImageNames = b;
}



