/***************************************************************************
                          kslidesaver.h  -  description
                             -------------------
    begin                : Fri Feb 21 00:14:47 CST 2003
    copyright            : (C) 2003 by Wojciech Jarosz
    email                : wjarosz@renderbtch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KSLIDESAVER_H
#define KSLIDESAVER_H

// this is needed to make libjpeg work with qt for some reason.
// qt defines INT32, and so does libjpeg, these two defines conflict
#define QT_CLEAN_NAMESPACE

// include files for QT
#include <qgl.h>
#include <vector>
#include <qstring.h>
#include <string>
#include "math/vec3.h"

class QTimer;
class QTime;
class imageLoader;

/*!
	The main openGL widget. This does all the work.
*/
class KSlideSaver : public QGLWidget
{
	Q_OBJECT        //!< must include this if you use Qt signals/slots

public:
	KSlideSaver(QWidget *parent = NULL, const char *name = NULL);
	~KSlideSaver();
	void init();

public slots:
	void setDelay(int delay);
	void setDirectory(const QString& directory);
	void setRandomize(bool b);
	void setFadeAmount(int percent);
	void setShowFPS(bool);
	void setShowImageNames(bool);

protected slots:
	void loadImage1();
	void loadImage2();
	void bindImage1(bool update = true);
	void bindImage2(bool update = true);
	void drawImage1();
	void drawImage2();
	void drawNoImage();
	void readConfig();
    
protected:
	void initializeGL();
	void resizeGL( int w, int h );
	void paintGL();
	void paintOverlays();
	void randomizeImages();
	void readImageDirectory();
	void addDirectory(const QString& dir, int depth);

	QTimer *idle_timer;
	QTime *start1, *start2;

	std::vector<std::string> fileNames;

	Vec3	startLoc1;
	Vec3	currentLoc1;
	Vec3	endLoc1;

	Vec3	startLoc2;
	Vec3	currentLoc2;
	Vec3	endLoc2;

	float opacity1, opacity2;
	float glImage1Width, glImage1Height;
	float glImage2Width, glImage2Height;

	GLuint texture[2];
	int redrawWait;
	int reloadWait;

	float startFade;
	bool image1Show, image2Show;

	QString imageDirectory;

	imageLoader* imageL1;
	imageLoader* imageL2;

	bool image1First;
	bool loadingImage1, loadingImage2;

	int image1Index, image2Index;
	bool randomize, starting;

	float w1Ratio, w2Ratio, h1Ratio, h2Ratio;
	int oldW1, oldH1, oldW2, oldH2;
	float windowAspect;
	bool showFPS;
	bool showImageNames;
	QTime *fpsTimer;
	QString image1Name, image2Name;
	double fps;

	int frameNum;
};

#endif 

