/*!	\file common.h
	\brief useful macros that are used everywhere

	This file is part of the renderBitch distribution.
	Copyright (C) 2002 Wojciech Jarosz

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef COMMON_H
#define COMMON_H

//!	A macro that holds the value of pi.
#ifndef M_PI
#	define M_PI			3.14159265358979323846264338327950288419716939937510582097494459
#endif

//!	A macro that holds the value of ln(2.0).
#ifndef LOG_2
#	define LOG_2		0.69314718056
#endif

//!	A macro that holds the value for a floating point epsilon.
#ifndef FEQ_EPS
#	define FEQ_EPS		1e-10
#endif

//!	A macro that holds the value for another floating point epsilon.
#ifndef FEQ_EPS2
#	define FEQ_EPS2		1e-10
#endif

//!	A macro that holds the value for positive infinity.
#ifndef FEQ_INF
#	define FEQ_INF		1.0e10
#endif

//!	A macro that tests if a floating point value is nearly Zero.
#ifndef IsZero
#	define IsZero(x)	(fabs(x)<FEQ_EPS)
#endif

//!	A macro that tests if two floating point values are nearly equal.
#ifndef FEQ
#	define FEQ(a, b)	(fabs((a)-(b))<FEQ_EPS)
#endif

#define		DEGREES_TO_RADIANS 0.0174532925199432957692369076848
#define		RADIANS_TO_DEGREES 57.295779513082320876798154814114

//!	A macro that converts from degrees to radians.
#define		TO_RADIANS(x)	((x) * DEGREES_TO_RADIANS)

//!	A macro that converts from radians to degrees.
#define		TO_DEGREES(x)	((x) * RADIANS_TO_DEGREES)

//!	A macro that returns the maximum of \a a and \a b.
#ifndef MAX
#	define MAX(a,b)			(((a) > (b)) ? (a) : (b))
#endif

//!	A macro that returns the minimum of \a a and \a b.
#ifndef MIN
#	define MIN(a,b)			(((a) < (b)) ? (a) : (b))
#endif

// COMMON_H
#endif
