/***************************************************************************
                          jpegloader.cpp  -  description
                             -------------------
    begin                : Tue Mar 4 2003
    copyright            : (C) 2003 by Wojciech Jarosz
    email                : wjarosz@renderbtch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "jpegloader.h"
#include "byteraster.h"

jpegLoader::jpegLoader() :
		imageLoader(),
		row_stride(1)
{

}

jpegLoader::~jpegLoader()
{
}


bool jpegLoader::read_image(const char * filename)
{
	infile = fopen(filename, "rb");
	if(!infile)
	{
		std::cerr << "jpegLoader: Cannot open file \"" << filename << "\"" << std::endl;
		delete img;
		img = NULL;
		return false;
	}

	cinfo.err = jpeg_std_error(&jerr);

	jpeg_create_decompress(&cinfo);
	jpeg_stdio_src(&cinfo, infile);

	jpeg_read_header(&cinfo, TRUE);
	jpeg_start_decompress(&cinfo);

	delete img;
	img = new ByteRaster(	cinfo.output_width,
							cinfo.output_height,
							cinfo.output_components);

	row_stride = cinfo.output_width * cinfo.output_components;
	scanline = img->head();
	while(cinfo.output_scanline < cinfo.output_height)
	{
		jpeg_read_scanlines(&cinfo, (JSAMPLE **)&scanline, 1);
		scanline += row_stride;
	}

	jpeg_finish_decompress(&cinfo);
	jpeg_destroy_decompress(&cinfo);
	fclose(infile);
	
	return true;
}

bool jpegLoader::start_incremental_image_read(const char * filename)
{
	if (reading)
		return false;
	reading = true;

	infile = fopen(filename, "rb");
	if(!infile)
	{
		std::cerr << "jpegLoader: Cannot open file \"" << filename << "\"" << std::endl;
		delete img;
		img = NULL;
		return false;
	}

	cinfo.err = jpeg_std_error(&jerr);

	jpeg_create_decompress(&cinfo);
	jpeg_stdio_src(&cinfo, infile);

	jpeg_read_header(&cinfo, TRUE);
	jpeg_start_decompress(&cinfo);

	delete img;
	img = new ByteRaster(	cinfo.output_width,
						 	cinfo.output_height,
						 	cinfo.output_components);

	row_stride = cinfo.output_width * cinfo.output_components;
	scanline = img->head();

	return true;
}

int jpegLoader::incremental_image_read(const int& numLines)
{
	int i;
	for (i = 0; cinfo.output_scanline < cinfo.output_height && i < numLines; i++)
	{
		jpeg_read_scanlines(&cinfo, (JSAMPLE **)&scanline, 1);
		scanline += row_stride;
	}

	return i;
}

void jpegLoader::finish_incremental_image_read(void)
{
	// finish up the image if we still haven't loaded everything yet
	while(cinfo.output_scanline < cinfo.output_height)
	{
		jpeg_read_scanlines(&cinfo, (JSAMPLE **)&scanline, 1);
		scanline += row_stride;
	}
	
	jpeg_finish_decompress(&cinfo);
	jpeg_destroy_decompress(&cinfo);
	fclose(infile);

	reading = false;
}
